/*  Lectura de targes RFID amb Arduino i lectorRFID MIFARE RC522 
    amb presentació dels resultats per pantalla Oled SDD1306.
    S'utilitzen dos tags, un clauer (ID: 3e9855d3) i una tarja (ID: b0859225)
    Utilitzem els següents ports:
    RST           D9
    SDA(SS)      D10
    MOSI         D11
    MISO         D12
    SCK          D13                                        */
 
#include <SPI.h>
#include <MFRC522.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

String identificatiu; 
const int pin_rst = 9;                              // configuració del pin del reset del RC522
const int pin_sda = 10;                             // configuració del pin del SS (SDA) del RC522
MFRC522 lectorRFID(pin_sda, pin_rst);               // crear instancia del MFRC522
 
#define OLED_RESET 4
Adafruit_SSD1306 pantallaOLED(OLED_RESET);
 
#define NUMFLAKES 10
#define XPOS 0
#define YPOS 1
#define DELTAY 2
 
void setup()
{
   SPI.begin();                                     // inicialització del bus SPI
   pantallaOLED.begin(SSD1306_SWITCHCAPVCC, 0x3C);  // inicialització de la pantalla OLED amb I2C adreça 0x3D (configuració 128x64)
   lectorRFID.PCD_Init();                           // inicialització del RFID
   pantallaOLED.clearDisplay();                     // esborra el logo d'Adafruit que porta la llibreria
   pantallaOLED.setTextSize(2);                     // presentació del missatge inicial
   pantallaOLED.setTextColor(WHITE);
   pantallaOLED.setCursor(0,10);
   pantallaOLED.print("Lectura ID");
   pantallaOLED.setCursor(0,30);
   pantallaOLED.print("Tarja RFID");
   pantallaOLED.setCursor(20,50);
   pantallaOLED.print("MFRC522");
   pantallaOLED.display();
}
 
void loop()
{
  String identificatiu = "";                             // inicialització de la variable
  if (lectorRFID.PICC_IsNewCardPresent())                // detectar una tarja nova
  {
    if (lectorRFID.PICC_ReadCardSerial())                // selecionar una tarja concreta
    {
      for ( uint8_t i = 0; i < lectorRFID.uid.size; i++) // lectura del buffer
      {
      uint8_t _baux=(uint8_t)lectorRFID.uid.uidByte[i];
      identificatiu+=String((uint8_t)_baux, HEX);
      }
      pantallaOLED.clearDisplay();                       // presentació de dades per pantalla
      pantallaOLED.setTextSize(1);
      pantallaOLED.setTextColor(WHITE);
      pantallaOLED.setCursor(0,5);      
      pantallaOLED.print("ID de la tarja:");
      pantallaOLED.setTextSize(2);
      pantallaOLED.setTextColor(WHITE);
      pantallaOLED.setCursor(0,15);         
      pantallaOLED.print(identificatiu);
      pantallaOLED.setTextSize(1);
      pantallaOLED.setTextColor(WHITE);
      pantallaOLED.setCursor(0,40);                  
      pantallaOLED.print("Estat de la tarja:");
      if (String(identificatiu).equals("3e9855d3")) {
        pantallaOLED.setTextSize(2);
        pantallaOLED.setTextColor(WHITE);
        pantallaOLED.setCursor(0,50);                  
        pantallaOLED.print("VALIDA");
      } else {
        pantallaOLED.setTextSize(2);
        pantallaOLED.setTextColor(WHITE);
        pantallaOLED.setCursor(0,50);                  
        pantallaOLED.print("NO VALIDA");
      }
      pantallaOLED.display();                            // executa la presentació per pantalla
      lectorRFID.PICC_HaltA();                           // finalitzar la lectura actual
      }
   }
   delay(250);
}
